; RGBooze v1.0
;
; Copyright (C) 2009 Paul Wilhelm <paul@mosfetkiller.de>
; http://mosfetkiller.de/?s=rgbooze

.include "tn12def.inc"						; ATtiny12-spezifische Definitionen
											; (kompatibel mit ATtiny11)
; Pins am AVR
.equ RED = PB0								; Rot
.equ GREEN = PB4							; Grün
.equ BLUE = PB2								; Blau

.def PERIOD_COUNTER = r0					; Periodenzähler

.def RED_COUNTER = r1						; PWM-Zähler
.def GREEN_COUNTER = r2
.def BLUE_COUNTER = r3

.def temp1 = r16							; Temporär
.def temp2 = r17

.def RED_PWM = r18							; Helligkeit
.def GREEN_PWM = r19
.def BLUE_PWM = r20

.def TIME_COUNTER_L = r21					; 16 Bit-Zeitzähler
.def TIME_COUNTER_H = r22

.def FADING_COMBINATION = r23				; Fading
.def FADING_DIR = r24

.def SWITCH_COUNTER = r25					; Tastenzähler

.equ PERIOD_LENGTH = 100					; Länge einer PWM-Periode
.equ FADING_DELAY = 255						; Delay zum Regeln der Fading-
											; Geschwindigkeit (maximal)

.cseg
.org 0
	rjmp	RESET							; Reset Handler
	rjmp	EXT_INT0						; IRQ0 Handler
	reti									; Pin Change Handler
	rjmp	TIM0_OVF						; Timer0 Overflow Handler
	reti									; EEPROM Ready Handler
	reti									; Analog Comparator Handler


; IRQ0 Handler = Aufwachpunkt
EXT_INT0:
	reti


; Timer0 Overflow Handler = Software-PWM
TIM0_OVF:
	inc		TIME_COUNTER_L					; Zeitzähler
	clr		temp1
	adc		TIME_COUNTER_H, temp1

	; PWM-Zähler
	DEC_RED:								; Rot
		dec		RED_COUNTER
		brne	DEC_GREEN

		cbi		PORTB, RED

	DEC_GREEN:								; Grün
		dec		GREEN_COUNTER
		brne	DEC_BLUE

		cbi		PORTB, GREEN

	DEC_BLUE:								; Blau
		dec		BLUE_COUNTER
		brne	DEC_END

		cbi		PORTB, BLUE

	DEC_END:

	dec		PERIOD_COUNTER					; Ende der Periode erreicht?
	tst		PERIOD_COUNTER
	breq	RESET_PWM

	reti


; Software-PWM Reset
RESET_PWM:
	ldi		temp1, PERIOD_LENGTH			; Neue Periode
	mov		PERIOD_COUNTER, temp1

	mov		RED_COUNTER, RED_PWM			; PWM-Zähler laden
	mov		GREEN_COUNTER, GREEN_PWM
	mov		BLUE_COUNTER, BLUE_PWM

	; LEDs setzen falls jeweiliger pwm-Wert > 0
	SET_RED:								; Rot
		tst		RED_PWM
		breq	SET_GREEN
		sbi		PORTB, RED

	SET_GREEN:								; Grün
		tst		GREEN_PWM
		breq	SET_BLUE
		sbi		PORTB, GREEN

	SET_BLUE:								; Blau
		tst		BLUE_PWM
		breq	SET_END
		sbi		PORTB, BLUE

	SET_END:

	reti


; Initialisierungen
RESET:
	; PORTB
	ldi		temp1, (1 << RED) | (1 << GREEN) | (1 << BLUE)
	out		DDRB, temp1

	ldi		temp1, (1 << PB1)				; Pullup für An-/Austaster
	out		PORTB, temp1

	; Timer0
	ldi		temp1, 0b001					; CK
	out		TCCR0, temp1

	ldi		temp1, (1 << TOIE0)				; Enable
	out		TIMSK, temp1

	; Software-PWM
	ldi		temp1, PERIOD_LENGTH
	mov		PERIOD_COUNTER, temp1

	ldi		RED_PWM, 0
	ldi		GREEN_PWM, 0
	ldi		BLUE_PWM, 0

	; Fading
	ldi		FADING_COMBINATION, (1 << RED)
	ldi		FADING_DIR, (1 << RED) | (1 << GREEN) | (1 << BLUE)

	clr		SWITCH_COUNTER

	sei


; Hauptprogramm
MAIN:


	; Hauptprogramm oder AVR schlafen legen?
	sbic	PINB, PB1
	rjmp	DELAY


	; Schlafen
	cli

	clr		temp1							; Timer0 aus
	out		TIMSK, temp1

	ldi		temp1, (1 << PB1)				; LEDs aus (PullUp für An-/Austaster)
	out		PORTB, temp1

	ldi		temp1, 200
	SWITCH_DELAY00:
		ldi		temp2, 255
		clz
		SWITCH_DELAY10:						; Warteschleife (Taster entprellen)
			dec		temp2
			brne	SWITCH_DELAY10

		dec		temp1
		brne	SWITCH_DELAY00

	WAIT0:									; Warten, bis Taste losgelassen wurde
		sbis	PINB, PB1
		rjmp	WAIT0

	ldi		temp1, 200
	SWITCH_DELAY01:
		ldi		temp2, 255
		clz
		SWITCH_DELAY11:						; Warteschleife (Taster entprellen)
			dec		temp2
			brne	SWITCH_DELAY11

		dec		temp1
		brne	SWITCH_DELAY01

	ldi		temp1, (1 << SM) | (1 << SE)	; Power-down Mode
	out		MCUCR, temp1

	ldi		temp1, (1 << INT0)				; External Interrupt Request 0 Enable
	out		GIMSK, temp1

	sei
	sleep									; Gute Nacht...
	cli

	clr		temp1							; External Interrupt Request 0 ausschalten
	out		GIMSK, temp1

	ldi		temp1, 200
	SWITCH_DELAY02:
		ldi		temp2, 255
		clz
		SWITCH_DELAY12:						; Warteschleife (Taster entprellen)
			dec		temp2
			brne	SWITCH_DELAY12

		dec		temp1
		brne	SWITCH_DELAY02

	WAIT1:									; Warten, bis Taste losgelassen wurde
		sbis	PINB, PB1
		rjmp	WAIT1

	ldi		temp1, 200
	SWITCH_DELAY03:
		ldi		temp2, 255
		clz
		SWITCH_DELAY13:						; Warteschleife (Taster entprellen)
			dec		temp2
			brne	SWITCH_DELAY13

		dec		temp1
		brne	SWITCH_DELAY03

	ldi		temp1, (1 << WDE)				; Reset provozieren (Watchdog)
	out		WDTCR, temp1

	DO_NOTHING:
		rjmp	DO_NOTHING


	; Delay
DELAY:
	cpi		TIME_COUNTER_H, FADING_DELAY
	brne	MAIN

	ldi		TIME_COUNTER_H, 0				; Zeitzähler resetten


	; Rot faden
FADE_RED:
	sbrs	FADING_COMBINATION, RED
	rjmp	FADE_RED_ZERO

	sbrs	FADING_DIR, RED
	rjmp	FADE_RED_DEC

	inc		RED_PWM							; Inkrementieren
	rjmp	FADE_RED_CHECK_OVF

FADE_RED_DEC:								; Dekrementieren
	dec		RED_PWM

FADE_RED_CHECK_OVF:							; Auf Überlauf prüfen
	cpi		RED_PWM, PERIOD_LENGTH
	brne	FADE_RED_CHECK_UVF

	andi	FADING_DIR, 255 - (1 << RED)	; Direction-Flag löschen
	rjmp	FADE_RED_END

FADE_RED_CHECK_UVF:							; Auf Unterlauf prüfen
	tst		RED_PWM
	brne	FADE_RED_END

	ori		FADING_DIR, (1 << RED)			; Direction-Flag setzen
	rjmp	FADE_RED_END

FADE_RED_ZERO:								; Zähler gegen 0 laufen lassen
	tst		RED_PWM
	breq	FADE_RED_END
	dec		RED_PWM

FADE_RED_END:


	; Grün faden
FADE_GREEN:
	sbrs	FADING_COMBINATION, GREEN
	rjmp	FADE_GREEN_ZERO

	sbrs	FADING_DIR, GREEN
	rjmp	FADE_GREEN_DEC

	inc		GREEN_PWM						; Inkrementieren
	rjmp	FADE_GREEN_CHECK_OVF

FADE_GREEN_DEC:								; Dekrementieren
	dec		GREEN_PWM

FADE_GREEN_CHECK_OVF:						; Auf Überlauf prüfen
	cpi		GREEN_PWM, PERIOD_LENGTH
	brne	FADE_GREEN_CHECK_UVF

	andi	FADING_DIR, 255 - (1 << GREEN)	; Direction-Flag löschen
	rjmp	FADE_GREEN_END

FADE_GREEN_CHECK_UVF:						; Auf Unterlauf prüfen
	tst		GREEN_PWM
	brne	FADE_GREEN_END

	ori		FADING_DIR, (1 << GREEN)		; Direction-Flag setzen
	rjmp	FADE_GREEN_END

FADE_GREEN_ZERO:							; Zähler gegen 0 laufen lassen
	tst		GREEN_PWM
	breq	FADE_GREEN_END
	dec		GREEN_PWM

FADE_GREEN_END:


	; Blau faden
FADE_BLUE:
	sbrs	FADING_COMBINATION, BLUE
	rjmp	FADE_BLUE_ZERO

	sbrs	FADING_DIR, BLUE
	rjmp	FADE_BLUE_DEC

	inc		BLUE_PWM						; Inkrementieren
	rjmp	FADE_BLUE_CHECK_OVF

FADE_BLUE_DEC:								; Dekrementieren
	dec		BLUE_PWM

FADE_BLUE_CHECK_OVF:						; Auf Überlauf prüfen
	cpi		BLUE_PWM, PERIOD_LENGTH
	brne	FADE_BLUE_CHECK_UVF

	andi	FADING_DIR, 255 - (1 << BLUE)	; Direction-Flag löschen
	rjmp	FADE_BLUE_END

FADE_BLUE_CHECK_UVF:						; Auf Unterlauf prüfen
	tst		BLUE_PWM
	brne	FADE_BLUE_END

	ori		FADING_DIR, (1 << BLUE)			; Direction-Flag setzen
	rjmp	FADE_BLUE_END

FADE_BLUE_ZERO:								; Zähler gegen 0 laufen lassen
	tst		BLUE_PWM
	breq	FADE_BLUE_END
	dec		BLUE_PWM

FADE_BLUE_END:


	; Farben der Reihe nach durchwechseln
	cpi		RED_PWM, PERIOD_LENGTH
	brne	CHECK_RED_END
	ldi		FADING_COMBINATION, (1 << GREEN)
	ori		FADING_DIR, (1 << GREEN)		; Direction-Flag setzen
CHECK_RED_END:

	cpi		GREEN_PWM, PERIOD_LENGTH
	brne	CHECK_GREEN_END
	ldi		FADING_COMBINATION, (1 << BLUE)
	ori		FADING_DIR, (1 << BLUE)			; Direction-Flag setzen
CHECK_GREEN_END:

	cpi		BLUE_PWM, PERIOD_LENGTH
	brne	CHECK_BLUE_END
	ldi		FADING_COMBINATION, (1 << RED)
	ori		FADING_DIR, (1 << RED)			; Direction-Flag setzen
CHECK_BLUE_END:


	rjmp	MAIN
